#include "cfglr_log.h"

#define TAG "CFGLR_NVS"

void cfglr_backend_nvs_open(cfglr_backend_t *backend, cfglr_handle_t *handle) {
  CFGLR_LOGI(TAG, "opening NVS partition: %s", handle->namespace);
  esp_err_t err = nvs_open(handle->namespace, NVS_READWRITE, (nvs_handle_t*)&backend->handle);
}

void cfglr_backend_nvs_close(cfglr_backend_t *backend, cfglr_handle_t *handle) {
}


uint16_t cfglr_backend_nvs_get(cfglr_backend_t *backend, cfglr_element_t *element, cfglr_handle_t *handle) {
  esp_err_t err = ESP_OK;

  if(element->datatype <= CFGLR_DATATYPE_I32) {
    switch(element->datatype) {
      case CFGLR_DATATYPE_U8:
        err = nvs_get_u8((nvs_handle_t)backend->handle, element->key, (uint8_t*)element->data);
      break;

      case CFGLR_DATATYPE_I8:
        err = nvs_get_i8((nvs_handle_t)backend->handle, element->key, (int8_t*)element->data);
      break;

      /*
      case CFGLR_DATATYPE_U16:
        err = nvs_get_u16((nvs_handle_t)backend->handle, element->key, (uint16_t*)element->data);
      break;

      case CFGLR_DATATYPE_I16:
        err = nvs_get_i16((nvs_handle_t)backend->handle, element->key, (int16_t*)element->data);
      break;

      case CFGLR_DATATYPE_U32:
        err = nvs_get_u32((nvs_handle_t)backend->handle, element->key, (uint32_t*)element->data);
      break;

      case CFGLR_DATATYPE_I32:
        err = nvs_get_i32((nvs_handle_t)backend->handle, element->key, (int32_t*)element->data);
      break;

      case CFGLR_DATATYPE_U64:
        err = nvs_get_u64((nvs_handle_t)backend->handle, element->key, (uint64_t*)element->data);
      break;

      case CFGLR_DATATYPE_I64:
        err = nvs_get_i64((nvs_handle_t)backend->handle, element->key, (int64_t*)element->data);
      break;
      */

      default:
        CFGLR_LOGI(TAG, "unhandlable datatype: %d", element->datatype);
    }

    if(err == ESP_ERR_NVS_NOT_FOUND) {
    } else if(err != ESP_OK) {
      ESP_ERROR_CHECK(err);
    } else if(err == ESP_OK) {
      element->dirty = 0;
    }
  } else {
    // handle [UI]64, STR and BIN datatype
  }

  return err;
}

uint16_t cfglr_backend_nvs_set(cfglr_backend_t *backend, cfglr_element_t *element, cfglr_handle_t *handle) {
  esp_err_t err = ESP_OK;

  switch(element->datatype) {
    case CFGLR_DATATYPE_U8:
      err = nvs_set_u8((nvs_handle_t)backend->handle, element->key, (uint8_t)element->data);
    break;

    case CFGLR_DATATYPE_I8:
      err = nvs_set_i8((nvs_handle_t)backend->handle, element->key, (int8_t)element->data);
    break;

    case CFGLR_DATATYPE_U16:
      err = nvs_set_u16((nvs_handle_t)backend->handle, element->key, (uint16_t)element->data);
    break;

    case CFGLR_DATATYPE_I16:
      err = nvs_set_i16((nvs_handle_t)backend->handle, element->key, (int16_t)element->data);
    break;

    case CFGLR_DATATYPE_U32:
      err = nvs_set_u32((nvs_handle_t)backend->handle, element->key, (uint32_t)element->data);
    break;

    case CFGLR_DATATYPE_I32:
      err = nvs_set_i32((nvs_handle_t)backend->handle, element->key, (int32_t)element->data);
    break;

    case CFGLR_DATATYPE_U64:
      err = nvs_set_u64((nvs_handle_t)backend->handle, element->key, (uint64_t)element->data);
    break;

    case CFGLR_DATATYPE_I64:
      err = nvs_set_i64((nvs_handle_t)backend->handle, element->key, (int64_t)element->data);
    break;

    case CFGLR_DATATYPE_STR:
    case CFGLR_DATATYPE_BIN:
      // determine length
      // fetch data by length
    break;

    default:
      CFGLR_LOGI(TAG, "unhandlable datatype: %d", element->datatype);
  }

  if(err == ESP_OK) {
  }

  return err;
}
