#include "string.h"
#include "configulator.h"
#include "cfglr_log.h"

#define TAG "CFGLR"

uint8_t cfglr_init(cfglr_handle_t *handle) {
  uint8_t tick = 0;
  cfglr_backend_t *backend;
  cfglr_element_t *element;
  cfglr_signaler_t *signaler;

  handle->backend_count = 0;
  handle->element_count = 0;

  CFGLR_LOGI(TAG, "initializing backends");

  while((backend = &handle->backends[tick])) {
    if(backend->open == NULL) break;

    backend->open(backend, handle);
    // do error validation

    ++handle->backend_count;
    ++tick;
  }

  tick = 0;

  CFGLR_LOGI(TAG, "initializing signalers");

  while((signaler = &handle->signalers[tick])) {
    if(signaler->dispatch == NULL) break;

    if(signaler->init != NULL) {
      signaler->init(signaler, handle);
    }

    ++handle->signaler_count;
    ++tick;
  }

  tick = 0;

  CFGLR_LOGI(TAG, "initializing elements");

  while((element = &handle->elements[tick])) {
    if(element->key == NULL) break;

    CFGLR_LOGD(TAG, "Initializing element: %s", element->key);
    CFGLR_LOGD(TAG, "size: %d", element->datatype_size);
    CFGLR_LOGD(TAG, "default: %d", (int)element->default_data);

    element->data = (void*)malloc(element->datatype_size);

    for(uint8_t i = 0; i < handle->backend_count; i++) {
      if(handle->backends[i].get(&handle->backends[i], element, handle) == 0) {
        for(uint8_t j = 0; j < handle->signaler_count; j++) {
          handle->signalers[i].dispatch(&handle->signalers[i], element, CFGLR_SIGNAL_LOADED);
        }
      }
    }

    ++handle->element_count;
    ++tick;
  }

  return 0;
}

uint16_t cfglr_commit_data(cfglr_backend_t *backend, cfglr_element_t *element, cfglr_handle_t *handle) {
  uint16_t ret = backend->set(backend, element, handle);

  if(ret == 0) {
    for(uint8_t i = 0; i < handle->signaler_count; i++) {
      handle->signalers[i].dispatch(&handle->signalers[i], element, CFGLR_SIGNAL_CHANGE);
    }
  }

  return ret;
}
